Class: Sentry::Hub

Inherits:
Object
  • Object
show all
Includes:
ArgumentCheckingHelper, CallbackHelper
Defined in:
lib/sentry/hub.rb

Defined Under Namespace

Classes: Layer

Constant Summary collapse

MUTEX =
Mutex.new

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(client, scope) ⇒ Hub

Returns a new instance of Hub.



18
19
20
21
22
23
# File 'lib/sentry/hub.rb', line 18

def initialize(client, scope)
  first_layer = Layer.new(client, scope)
  @stack = [first_layer]
  @last_event_id = nil
  @current_profiler = {}
end

Instance Attribute Details

#current_profiler ⇒ Object (readonly)

Returns the value of attribute current_profiler.



16
17
18
# File 'lib/sentry/hub.rb', line 16

def current_profiler
  @current_profiler
end

#last_event_id ⇒ Object (readonly)

Returns the value of attribute last_event_id.



14
15
16
# File 'lib/sentry/hub.rb', line 14

def last_event_id
  @last_event_id
end

Instance Method Details

#add_breadcrumb(breadcrumb, hint: {}) ⇒ Object



295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
# File 'lib/sentry/hub.rb', line 295

def add_breadcrumb(breadcrumb, hint: {})
  return unless current_client
  return unless configuration.enabled_in_current_env?

  if before_breadcrumb = current_client.configuration.before_breadcrumb
    breadcrumb = safe_dispatch_callback(
      "before_breadcrumb",
      before_breadcrumb,
      [breadcrumb, hint]
    )
  end

  return unless breadcrumb

  current_scope.add_breadcrumb(breadcrumb)
end

#bind_client(client) ⇒ Object



82
83
84
85
86
87
88
# File 'lib/sentry/hub.rb', line 82

def bind_client(client)
  layer = current_layer

  if layer
    layer.client = client
  end
end

#capture_check_in(slug, status, **options) ⇒ Object



203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
# File 'lib/sentry/hub.rb', line 203

def capture_check_in(slug, status, **options)
  check_argument_type!(slug, ::String)
  check_argument_includes!(status, Sentry::CheckInEvent::VALID_STATUSES)

  return unless current_client

  options[:hint] ||= {}
  options[:hint][:slug] = slug

  event = current_client.event_from_check_in(
    slug,
    status,
    options[:hint],
    duration: options.delete(:duration),
    monitor_config: options.delete(:monitor_config),
    check_in_id: options.delete(:check_in_id)
  )

  return unless event

  capture_event(event, **options)
  event.check_in_id
end

#capture_event(event, **options, &block) ⇒ Object



262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
# File 'lib/sentry/hub.rb', line 262

def capture_event(event, **options, &block)
  check_argument_type!(event, Sentry::Event)

  return unless current_client

  hint = options.delete(:hint) || {}
  scope = current_scope.dup

  if block
    block.call(scope)
  elsif custom_scope = options[:scope]
    scope.update_from_scope(custom_scope)
  elsif !options.empty?
    unsupported_option_keys = scope.update_from_options(**options)

    unless unsupported_option_keys.empty?
      configuration.log_debug <<~MSG
        Options #{unsupported_option_keys} are not supported and will not be applied to the event.
        You may want to set them under the `extra` option.
      MSG
    end
  end

  event = current_client.capture_event(event, scope, hint)

  if event && configuration.debug
    configuration.log_debug(event.to_json_compatible)
  end

  @last_event_id = event&.event_id if event.is_a?(Sentry::ErrorEvent)
  event
end

#capture_exception(exception, **options, &block) ⇒ Object



162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
# File 'lib/sentry/hub.rb', line 162

def capture_exception(exception, **options, &block)
  if RUBY_PLATFORM == "java"
    check_argument_type!(exception, ::Exception, ::Java::JavaLang::Throwable)
  else
    check_argument_type!(exception, ::Exception)
  end

  return if Sentry.exception_captured?(exception)

  return unless current_client

  options[:hint] ||= {}
  options[:hint][:exception] = exception

  event = current_client.event_from_exception(exception, options[:hint])

  return unless event

  current_scope.session&.update_from_exception(event.exception)

  capture_event(event, **options, &block).tap do
    # mark the exception as captured so we can use this information to avoid duplicated capturing
    exception.instance_variable_set(Sentry::CAPTURED_SIGNATURE, true)
  end
end

#capture_log_event(message, **options) ⇒ Object



227
228
229
230
231
232
233
234
235
# File 'lib/sentry/hub.rb', line 227

def capture_log_event(message, **options)
  return unless current_client

  event = current_client.event_from_log(message, **options)

  return unless event

  current_client.buffer_log_event(event, current_scope)
end

#capture_message(message, **options, &block) ⇒ Object



188
189
190
191
192
193
194
195
196
197
198
199
200
201
# File 'lib/sentry/hub.rb', line 188

def capture_message(message, **options, &block)
  check_argument_type!(message, ::String)

  return unless current_client

  options[:hint] ||= {}
  options[:hint][:message] = message
  backtrace = options.delete(:backtrace)
  event = current_client.event_from_message(message, options[:hint], backtrace: backtrace)

  return unless event

  capture_event(event, **options, &block)
end

#capture_metric(name:, type:, value:, unit: nil, attributes: nil, integration: nil) ⇒ void

This method returns an undefined value.

Captures a metric and sends it to Sentry

Parameters:

  • name (String) —

    the metric name

  • type (Symbol) —

    the metric type (:counter, :gauge, :distribution)

  • value (Numeric) —

    the metric value

  • unit (String, nil) (defaults to: nil) —

    (optional) the metric unit

  • attributes (Hash, nil) (defaults to: nil) —

    (optional) additional attributes for the metric

  • integration (String, Symbol, nil) (defaults to: nil) —

    (optional) registered integration emitting the metric



246
247
248
249
250
251
252
253
254
255
256
257
258
259
# File 'lib/sentry/hub.rb', line 246

def capture_metric(name:, type:, value:, unit: nil, attributes: nil, integration: nil)
  return unless current_client

  metric = MetricEvent.new(
    name: name,
    value: value,
    type: type,
    unit: unit,
    attributes: attributes&.dup,
    integration_meta: Sentry.integrations[integration.to_s]
  )

  current_client.buffer_metric_event(metric, current_scope)
end

#clients ⇒ Array<Client>

All clients bound across the hub's scope stack, base layer first.

Returns:



60
61
62
# File 'lib/sentry/hub.rb', line 60

def clients
  @stack.map(&:client).compact
end

#clone ⇒ Object



72
73
74
75
76
77
78
79
80
# File 'lib/sentry/hub.rb', line 72

def clone
  layer = current_layer

  if layer
    scope = layer.scope&.dup

    Hub.new(layer.client, scope)
  end
end

#configuration ⇒ Object



64
65
66
# File 'lib/sentry/hub.rb', line 64

def configuration
  current_client.configuration
end

#configure_scope(&block) ⇒ Object



90
91
92
# File 'lib/sentry/hub.rb', line 90

def configure_scope(&block)
  block.call(current_scope)
end

#continue_trace(env, **options) ⇒ Object



383
384
385
386
387
388
389
390
391
392
393
394
395
396
397
398
399
# File 'lib/sentry/hub.rb', line 383

def continue_trace(env, **options)
  configure_scope { |s| s.generate_propagation_context(env) }

  return nil unless configuration.tracing_enabled?

  propagation_context = current_scope.propagation_context
  return nil unless propagation_context.incoming_trace

  Transaction.new(
    trace_id: propagation_context.trace_id,
    parent_span_id: propagation_context.parent_span_id,
    parent_sampled: propagation_context.parent_sampled,
    baggage: propagation_context.baggage,
    sample_rand: propagation_context.sample_rand,
    **options
  )
end

#current_client ⇒ Object



54
55
56
# File 'lib/sentry/hub.rb', line 54

def current_client
  current_layer&.client
end

#current_scope ⇒ Object



68
69
70
# File 'lib/sentry/hub.rb', line 68

def current_scope
  current_layer&.scope
end

#end_session ⇒ Object



328
329
330
331
332
333
334
335
336
337
338
339
340
# File 'lib/sentry/hub.rb', line 328

def end_session
  return unless current_scope
  session = current_scope.session
  current_scope.set_session(nil)

  return unless session
  session.close

  # NOTE: Under some circumstances, session_flusher nilified out of sync
  #   See: https://github.com/getsentry/sentry-ruby/issues/2378
  #   See: https://github.com/getsentry/sentry-ruby/pull/2396
  Sentry.session_flusher&.add_session(session)
end

#get_baggage ⇒ Object



358
359
360
361
362
363
# File 'lib/sentry/hub.rb', line 358

def get_baggage
  return nil unless current_scope

  current_scope.get_span&.to_baggage ||
    current_scope.propagation_context.get_baggage&.serialize
end

#get_trace_propagation_headers ⇒ Object



365
366
367
368
369
370
371
372
373
374
375
# File 'lib/sentry/hub.rb', line 365

def get_trace_propagation_headers
  headers = {}

  traceparent = get_traceparent
  headers[SENTRY_TRACE_HEADER_NAME] = traceparent if traceparent

  baggage = get_baggage
  headers[BAGGAGE_HEADER_NAME] = baggage if baggage && !baggage.empty?

  headers
end

#get_trace_propagation_meta ⇒ Object



377
378
379
380
381
# File 'lib/sentry/hub.rb', line 377

def get_trace_propagation_meta
  get_trace_propagation_headers.map do |k, v|
    "<meta name=\"#{k}\" content=\"#{v}\">"
  end.join("\n")
end

#get_traceparent ⇒ Object



351
352
353
354
355
356
# File 'lib/sentry/hub.rb', line 351

def get_traceparent
  return nil unless current_scope

  current_scope.get_span&.to_sentry_trace ||
    current_scope.propagation_context.get_traceparent
end

#new_from_top ⇒ Object



50
51
52
# File 'lib/sentry/hub.rb', line 50

def new_from_top
  Hub.new(current_client, current_scope)
end

#pop_scope ⇒ Object



112
113
114
115
116
117
118
119
120
# File 'lib/sentry/hub.rb', line 112

def pop_scope
  if @stack.size > 1
    @stack.pop
  else
    # We never want to enter a situation where we have no scope and no client
    client = current_client
    @stack = [Layer.new(client, Scope.new)]
  end
end

#profiler_running? ⇒ Boolean

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

This is an internal private method

Returns:

  • (Boolean)


44
45
46
47
48
# File 'lib/sentry/hub.rb', line 44

def profiler_running?
  MUTEX.synchronize do
    !@current_profiler.empty?
  end
end

#push_scope ⇒ Object



101
102
103
104
105
106
107
108
109
110
# File 'lib/sentry/hub.rb', line 101

def push_scope
  new_scope =
    if current_scope
      current_scope.dup
    else
      Scope.new
    end

  @stack << Layer.new(current_client, new_scope)
end

#start_profiler!(transaction) ⇒ Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

This is an internal private method



27
28
29
30
31
32
# File 'lib/sentry/hub.rb', line 27

def start_profiler!(transaction)
  MUTEX.synchronize do
    transaction.start_profiler!
    @current_profiler[transaction.__id__] = transaction.profiler
  end
end

#start_session ⇒ Object



323
324
325
326
# File 'lib/sentry/hub.rb', line 323

def start_session
  return unless current_scope
  current_scope.set_session(Session.new)
end

#start_transaction(transaction: nil, custom_sampling_context: {}, instrumenter: :sentry, **options) ⇒ Object



122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
# File 'lib/sentry/hub.rb', line 122

def start_transaction(transaction: nil, custom_sampling_context: {}, instrumenter: :sentry, **options)
  return unless configuration.tracing_enabled?
  return unless instrumenter == configuration.instrumenter

  transaction ||= Transaction.new(**options)

  sampling_context = {
    transaction_context: transaction.to_h,
    parent_sampled: transaction.parent_sampled,
    parent_sample_rate: transaction.parent_sample_rate
  }

  sampling_context.merge!(custom_sampling_context)
  transaction.set_initial_sample_decision(sampling_context: sampling_context)

  start_profiler!(transaction)

  transaction
end

#stop_profiler!(transaction) ⇒ Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

This is an internal private method



36
37
38
39
40
# File 'lib/sentry/hub.rb', line 36

def stop_profiler!(transaction)
  MUTEX.synchronize do
    @current_profiler.delete(transaction.__id__)&.stop
  end
end

#with_background_worker_disabled(&block) ⇒ Object

this doesn't do anything to the already initialized background worker but it temporarily disables dispatching events to it



314
315
316
317
318
319
320
321
# File 'lib/sentry/hub.rb', line 314

def with_background_worker_disabled(&block)
  original_background_worker_threads = configuration.background_worker_threads
  configuration.background_worker_threads = 0

  block.call
ensure
  configuration.background_worker_threads = original_background_worker_threads
end

#with_child_span(instrumenter: :sentry, **attributes, &block) ⇒ Object



142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
# File 'lib/sentry/hub.rb', line 142

def with_child_span(instrumenter: :sentry, **attributes, &block)
  return yield(nil) unless instrumenter == configuration.instrumenter

  current_span = current_scope.get_span
  return yield(nil) unless current_span

  result = nil

  begin
    current_span.with_child_span(**attributes) do |child_span|
      current_scope.set_span(child_span)
      result = yield(child_span)
    end
  ensure
    current_scope.set_span(current_span)
  end

  result
end

#with_scope(&block) ⇒ Object



94
95
96
97
98
99
# File 'lib/sentry/hub.rb', line 94

def with_scope(&block)
  push_scope
  yield(current_scope)
ensure
  pop_scope
end

#with_session_tracking(&block) ⇒ Object



342
343
344
345
346
347
348
349
# File 'lib/sentry/hub.rb', line 342

def with_session_tracking(&block)
  return yield unless configuration.session_tracking?

  start_session
  yield
ensure
  end_session
end