Module: Sentry::StdLibLogger

Includes:
CallbackHelper
Defined in:
lib/sentry/std_lib_logger.rb

Overview

Ruby Logger support Add commentMore actions intercepts any logger instance and send the log to Sentry too.

Constant Summary collapse

SEVERITY_MAP =
{
  0 => :debug,
  1 => :info,
  2 => :warn,
  3 => :error,
  4 => :fatal
}.freeze
ORIGIN =
"auto.log.ruby.std_logger"

Instance Method Summary collapse

Instance Method Details

#add(severity, message = nil, progname = nil, &block) ⇒ Object



19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
# File 'lib/sentry/std_lib_logger.rb', line 19

def add(severity, message = nil, progname = nil, &block)
  result = super

  return unless Sentry.initialized? && Sentry.get_current_hub

  # Only process logs that meet or exceed the logger's level
  return result if severity < level

  # exclude sentry SDK logs -- to prevent recursive log action,
  # do not process internal logs again
  if message.nil? && progname != Sentry::Logger::PROGNAME

    # handle different nature of Ruby Logger class:
    # inspo from Sentry::Breadcrumb::SentryLogger
    if block_given?
      message = yield
    else
      message = progname
    end

    message = message.to_s.strip

    if !message.nil? && message != Sentry::Logger::PROGNAME && method = SEVERITY_MAP[severity]
      if filter = Sentry.configuration.std_lib_logger_filter
        return result unless safe_dispatch_callback(
          "std_lib_logger_filter",
          filter,
          [self, message, method]
        )
      end

      Sentry.logger.send(method, message, origin: ORIGIN)
    end
  end

  result
end